@@ -375,6 +375,137 @@ Base<elem_type,derived>::is_hermitian(const typename get_pod_type<elem_type>::re
 
 
 
+template<typename elem_type, typename derived>
+inline
+arma_warn_unused
+bool
+Base<elem_type,derived>::is_zero(const typename get_pod_type<elem_type>::result tol) const
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename get_pod_type<elem_type>::result T;
+  
+  arma_debug_check( (tol < T(0)), "is_zero(): parameter 'tol' must be >= 0" );
+  
+  if(Proxy<derived>::use_at || is_Mat<typename Proxy<derived>::stored_type>::value)
+    {
+    const quasi_unwrap<derived> U( (*this).get_ref() );
+    
+    return arrayops::is_zero( U.M.memptr(), U.M.n_elem, tol );
+    }
+  
+  const Proxy<derived> P( (*this).get_ref() );
+  
+  const uword n_elem = P.get_n_elem();
+  
+  if(n_elem == 0)  { return false; }
+  
+  const typename Proxy<derived>::ea_type Pea = P.get_ea();
+  
+  if(is_cx<elem_type>::yes)
+    {
+    for(uword i=0; i<n_elem; ++i)
+      {
+      const elem_type val = Pea[i];
+      
+      const T val_real = access::tmp_real(val);
+      const T val_imag = access::tmp_imag(val);
+      
+      if(std::abs(val_real) > tol)  { return false; }
+      if(std::abs(val_imag) > tol)  { return false; }
+      }
+    }
+  else  // not complex
+    {
+    for(uword i=0; i<n_elem; ++i)
+      {
+      if(std::abs(Pea[i]) > tol)  { return false; }
+      }
+    }
+  
+  return true;
+  }
+
+
+
+template<typename elem_type, typename derived>
+inline
+arma_warn_unused
+bool
+Base<elem_type,derived>::is_trimatu() const
+  {
+  arma_extra_debug_sigprint();
+  
+  const quasi_unwrap<derived> U( (*this).get_ref() );
+  
+  if(U.M.n_rows != U.M.n_cols)  { return false; }
+  
+  if(U.M.n_elem <= 1)  { return true; }
+  
+  return trimat_helper::is_triu(U.M);
+  }
+
+
+
+template<typename elem_type, typename derived>
+inline
+arma_warn_unused
+bool
+Base<elem_type,derived>::is_trimatl() const
+  {
+  arma_extra_debug_sigprint();
+  
+  const quasi_unwrap<derived> U( (*this).get_ref() );
+  
+  if(U.M.n_rows != U.M.n_cols)  { return false; }
+  
+  if(U.M.n_elem <= 1)  { return true; }
+  
+  return trimat_helper::is_tril(U.M);
+  }
+
+
+
+template<typename elem_type, typename derived>
+inline
+arma_warn_unused
+bool
+Base<elem_type,derived>::is_diagmat() const
+  {
+  arma_extra_debug_sigprint();
+  
+  const quasi_unwrap<derived> U( (*this).get_ref() );
+  
+  const Mat<elem_type>& A = U.M;
+  
+  if(A.n_elem <= 1)  { return true; }
+  
+  // NOTE: we're NOT assuming the matrix has a square size
+  
+  const uword A_n_rows = A.n_rows;
+  const uword A_n_cols = A.n_cols;
+  
+  const elem_type* A_mem = A.memptr();
+  
+  if(A_mem[1] != elem_type(0))  { return false; }
+  
+  // if we got to this point, do a thorough check
+  
+  for(uword A_col=0; A_col < A_n_cols; ++A_col)
+    {
+    for(uword A_row=0; A_row < A_n_rows; ++A_row)
+      {
+      if( (A_mem[A_row] != elem_type(0)) && (A_row != A_col) )  { return false; }
+      }
+    
+    A_mem += A_n_rows;
+    }
+  
+  return true;
+  }
+
+
+
 template<typename elem_type, typename derived>
 inline
 arma_warn_unused
@@ -624,32 +755,6 @@ Base_extra_yes<elem_type, derived>::i() const
 
 
 
-template<typename elem_type, typename derived>
-arma_deprecated
-inline
-const Op<derived,op_inv>
-Base_extra_yes<elem_type, derived>::i(const bool) const   // argument kept only for compatibility with old user code
-  {
-  // arma_debug_warn(".i(bool) is deprecated and will be removed; change to .i()");
-  
-  return Op<derived,op_inv>(static_cast<const derived&>(*this));
-  }
-
-
-
-template<typename elem_type, typename derived>
-arma_deprecated
-inline
-const Op<derived,op_inv>
-Base_extra_yes<elem_type, derived>::i(const char*) const   // argument kept only for compatibility with old user code
-  {
-  // arma_debug_warn(".i(char*) is deprecated and will be removed; change to .i()");
-  
-  return Op<derived,op_inv>(static_cast<const derived&>(*this));
-  }
-
-
-
 template<typename elem_type, typename derived>
 inline
 arma_warn_unused
