@@ -53,7 +53,7 @@ expmat(Mat<typename T1::elem_type>& B, const Base<typename T1::elem_type,T1>& A)
   if(status == false)
     {
     arma_debug_warn("expmat(): given matrix appears ill-conditioned");
-    B.reset();
+    B.soft_reset();
     return false;
     }
   
@@ -90,7 +90,7 @@ expmat_sym(Mat<typename T1::elem_type>& Y, const Base<typename T1::elem_type,T1>
   
   if(status == false)
     {
-    Y.reset();
+    Y.soft_reset();
     arma_debug_warn("expmat_sym(): transformation failed");
     }
   
