@@ -18,59 +18,9 @@
 //! @{
 
 
-struct gmm_dist_mode { const uword id;  inline explicit gmm_dist_mode(const uword in_id) : id(in_id) {} };
-
-inline bool operator==(const gmm_dist_mode& a, const gmm_dist_mode& b) { return (a.id == b.id); }
-inline bool operator!=(const gmm_dist_mode& a, const gmm_dist_mode& b) { return (a.id != b.id); }
-
-struct gmm_dist_eucl : public gmm_dist_mode { inline gmm_dist_eucl() : gmm_dist_mode(1) {} };
-struct gmm_dist_maha : public gmm_dist_mode { inline gmm_dist_maha() : gmm_dist_mode(2) {} };
-struct gmm_dist_prob : public gmm_dist_mode { inline gmm_dist_prob() : gmm_dist_mode(3) {} };
-
-static const gmm_dist_eucl eucl_dist;
-static const gmm_dist_maha maha_dist;
-static const gmm_dist_prob prob_dist;
-
-
-
-struct gmm_seed_mode { const uword id; inline explicit gmm_seed_mode(const uword in_id) : id(in_id) {} };
-
-inline bool operator==(const gmm_seed_mode& a, const gmm_seed_mode& b) { return (a.id == b.id); }
-inline bool operator!=(const gmm_seed_mode& a, const gmm_seed_mode& b) { return (a.id != b.id); }
-
-struct gmm_seed_keep_existing : public gmm_seed_mode { inline gmm_seed_keep_existing() : gmm_seed_mode(1) {} };
-struct gmm_seed_static_subset : public gmm_seed_mode { inline gmm_seed_static_subset() : gmm_seed_mode(2) {} };
-struct gmm_seed_static_spread : public gmm_seed_mode { inline gmm_seed_static_spread() : gmm_seed_mode(3) {} };
-struct gmm_seed_random_subset : public gmm_seed_mode { inline gmm_seed_random_subset() : gmm_seed_mode(4) {} };
-struct gmm_seed_random_spread : public gmm_seed_mode { inline gmm_seed_random_spread() : gmm_seed_mode(5) {} };
-
-static const gmm_seed_keep_existing keep_existing;
-static const gmm_seed_static_subset static_subset;
-static const gmm_seed_static_spread static_spread;
-static const gmm_seed_random_subset random_subset;
-static const gmm_seed_random_spread random_spread;
-
-
 namespace gmm_priv
 {
 
-struct gmm_empty_arg {};
-
-
-#if defined(ARMA_USE_OPENMP)
-  struct arma_omp_state
-    {
-    const int orig_dynamic_state;
-    
-    inline  arma_omp_state() : orig_dynamic_state(omp_get_dynamic()) { omp_set_dynamic(0); }
-    inline ~arma_omp_state() { omp_set_dynamic(orig_dynamic_state); }
-    };
-#else
-  struct arma_omp_state {};
-#endif
-
-
-
 template<typename eT>
 class gmm_diag
   {
@@ -89,6 +39,9 @@ class gmm_diag
   inline            gmm_diag(const gmm_diag& x);
   inline gmm_diag& operator=(const gmm_diag& x);
   
+  inline explicit            gmm_diag(const gmm_full<eT>& x);
+  inline          gmm_diag& operator=(const gmm_full<eT>& x);
+  
   inline      gmm_diag(const uword in_n_dims, const uword in_n_gaus);
   inline void    reset(const uword in_n_dims, const uword in_n_gaus);
   inline void    reset();
@@ -109,17 +62,20 @@ class gmm_diag
   inline Col<eT> generate()              const;
   inline Mat<eT> generate(const uword N) const;
   
-  template<typename T1> inline eT      log_p(const T1& expr, const gmm_empty_arg& junk1 = gmm_empty_arg(), typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == true ))>::result* junk2 = 0) const;
-  template<typename T1> inline eT      log_p(const T1& expr, const uword gaus_id,                          typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == true ))>::result* junk2 = 0) const;
+  template<typename T1> inline eT      log_p(const T1& expr, const gmm_empty_arg& junk1 = gmm_empty_arg(), typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == true ))>::result* junk2 = nullptr) const;
+  template<typename T1> inline eT      log_p(const T1& expr, const uword gaus_id,                          typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == true ))>::result* junk2 = nullptr) const;
   
-  template<typename T1> inline Row<eT> log_p(const T1& expr, const gmm_empty_arg& junk1 = gmm_empty_arg(), typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == false))>::result* junk2 = 0) const;
-  template<typename T1> inline Row<eT> log_p(const T1& expr, const uword gaus_id,                          typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == false))>::result* junk2 = 0) const;
+  template<typename T1> inline Row<eT> log_p(const T1& expr, const gmm_empty_arg& junk1 = gmm_empty_arg(), typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == false))>::result* junk2 = nullptr) const;
+  template<typename T1> inline Row<eT> log_p(const T1& expr, const uword gaus_id,                          typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == false))>::result* junk2 = nullptr) const;
+  
+  template<typename T1> inline eT  sum_log_p(const Base<eT,T1>& expr)                      const;
+  template<typename T1> inline eT  sum_log_p(const Base<eT,T1>& expr, const uword gaus_id) const;
   
   template<typename T1> inline eT  avg_log_p(const Base<eT,T1>& expr)                      const;
   template<typename T1> inline eT  avg_log_p(const Base<eT,T1>& expr, const uword gaus_id) const;
   
-  template<typename T1> inline uword   assign(const T1& expr, const gmm_dist_mode& dist, typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == true ))>::result* junk = 0) const;
-  template<typename T1> inline urowvec assign(const T1& expr, const gmm_dist_mode& dist, typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == false))>::result* junk = 0) const;
+  template<typename T1> inline uword   assign(const T1& expr, const gmm_dist_mode& dist, typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == true ))>::result* junk = nullptr) const;
+  template<typename T1> inline urowvec assign(const T1& expr, const gmm_dist_mode& dist, typename enable_if<((is_arma_type<T1>::value) && (resolves_to_colvector<T1>::value == false))>::result* junk = nullptr) const;
   
   template<typename T1> inline urowvec  raw_hist(const Base<eT,T1>& expr, const gmm_dist_mode& dist_mode) const;
   template<typename T1> inline Row<eT> norm_hist(const Base<eT,T1>& expr, const gmm_dist_mode& dist_mode) const;
@@ -158,33 +114,37 @@ class gmm_diag
   
   protected:
   
-  
+  arma_aligned Mat<eT> inv_dcovs;
   arma_aligned Row<eT> log_det_etc;
   arma_aligned Row<eT> log_hefts;
   arma_aligned Col<eT> mah_aux;
   
   //
   
-  inline void init(const gmm_diag& x);
+  inline void init(const gmm_diag&     x);
+  inline void init(const gmm_full<eT>& x);
   
   inline void init(const uword in_n_dim, const uword in_n_gaus);
   
   inline void init_constants();
-
+  
   inline umat internal_gen_boundaries(const uword N) const;
-
+  
   inline eT internal_scalar_log_p(const eT* x                     ) const;
   inline eT internal_scalar_log_p(const eT* x, const uword gaus_id) const;
   
-  template<typename T1> inline Row<eT> internal_vec_log_p(const T1& X                     ) const;
-  template<typename T1> inline Row<eT> internal_vec_log_p(const T1& X, const uword gaus_id) const;
+  inline Row<eT> internal_vec_log_p(const Mat<eT>& X                     ) const;
+  inline Row<eT> internal_vec_log_p(const Mat<eT>& X, const uword gaus_id) const;
   
-  template<typename T1> inline eT internal_avg_log_p(const T1& X                     ) const;
-  template<typename T1> inline eT internal_avg_log_p(const T1& X, const uword gaus_id) const;
+  inline eT internal_sum_log_p(const Mat<eT>& X                     ) const;
+  inline eT internal_sum_log_p(const Mat<eT>& X, const uword gaus_id) const;
   
-  template<typename T1> inline uword internal_scalar_assign(const T1& X, const gmm_dist_mode& dist_mode) const;
+  inline eT internal_avg_log_p(const Mat<eT>& X                     ) const;
+  inline eT internal_avg_log_p(const Mat<eT>& X, const uword gaus_id) const;
   
-  template<typename T1> inline void internal_vec_assign(urowvec& out, const T1& X, const gmm_dist_mode& dist_mode) const;
+  inline uword internal_scalar_assign(const Mat<eT>& X, const gmm_dist_mode& dist_mode) const;
+  
+  inline void internal_vec_assign(urowvec& out, const Mat<eT>& X, const gmm_dist_mode& dist_mode) const;
   
   inline void internal_raw_hist(urowvec& hist, const Mat<eT>& X, const gmm_dist_mode& dist_mode) const;
   
@@ -192,12 +152,10 @@ class gmm_diag
   
   template<uword dist_id> inline void generate_initial_means(const Mat<eT>& X, const gmm_seed_mode& seed);
   
-  template<uword dist_id> inline void generate_initial_dcovs_and_hefts(const Mat<eT>& X, const eT var_floor);
+  template<uword dist_id> inline void generate_initial_params(const Mat<eT>& X, const eT var_floor);
   
   template<uword dist_id> inline bool km_iterate(const Mat<eT>& X, const uword max_iter, const bool verbose, const char* signature);
   
-  template<uword dist_id> inline void km_update_stats(const Mat<eT>& X, const uword start_index, const uword end_index, const Mat<eT>& old_means, field< running_mean_vec<eT> >& running_means) const;
-  
   //
   
   inline bool em_iterate(const Mat<eT>& X, const uword max_iter, const eT var_floor, const bool verbose);
