@@ -22,41 +22,29 @@ class memory
   {
   public:
   
-  arma_inline static uword enlarge_to_mult_of_chunksize(const uword n_elem);
-  
-  template<typename eT> inline arma_malloc static eT*         acquire(const uword n_elem);
-  template<typename eT> inline arma_malloc static eT* acquire_chunked(const uword n_elem);
+  template<typename eT> inline arma_malloc static eT* acquire(const uword n_elem);
   
   template<typename eT> arma_inline static void release(eT* mem);
   
   template<typename eT> arma_inline static bool      is_aligned(const eT*  mem);
   template<typename eT> arma_inline static void mark_as_aligned(      eT*& mem);
   template<typename eT> arma_inline static void mark_as_aligned(const eT*& mem);
+  
+  // deprecated functions that will be removed
+  
+  inline arma_deprecated static uword enlarge_to_mult_of_chunksize(const uword n_elem);         //! NOTE: do not use this function; it will be removed
+  template<typename eT> inline arma_deprecated static eT* acquire_chunked(const uword n_elem);  //! NOTE: do not use this function; it will be removed
   };
 
 
 
-arma_inline
-uword
-memory::enlarge_to_mult_of_chunksize(const uword n_elem)
-  {
-  const uword chunksize = arma_config::spmat_chunksize;
-  
-  // this relies on integer division
-  const uword n_elem_mod = (n_elem > 0) ? (((n_elem-1) / chunksize) + 1) * chunksize : uword(0);
-  
-  return n_elem_mod;
-  }
-
-
-
 template<typename eT>
 inline
 arma_malloc
 eT*
 memory::acquire(const uword n_elem)
   {
-  if(n_elem == 0)  { return NULL; }
+  if(n_elem == 0)  { return nullptr; }
   
   arma_debug_check
     (
@@ -72,18 +60,19 @@ memory::acquire(const uword n_elem)
     }
   #elif defined(ARMA_USE_MKL_ALLOC)
     {
-    out_memptr = (eT *) mkl_malloc( sizeof(eT)*n_elem, 128 );
+    out_memptr = (eT *) mkl_malloc( sizeof(eT)*n_elem, 32 );
     }
   #elif defined(ARMA_HAVE_POSIX_MEMALIGN)
     {
-    eT* memptr = NULL;
+    eT* memptr = nullptr;
     
     const size_t n_bytes   = sizeof(eT)*size_t(n_elem);
-    const size_t alignment = (n_bytes >= size_t(1024)) ? size_t(64) : size_t(16);
+    const size_t alignment = (n_bytes >= size_t(1024)) ? size_t(32) : size_t(16);
     
+    // TODO: investigate apparent memory leak when using alignment >= 64 (as shown on Fedora 28, glibc 2.27)
     int status = posix_memalign((void **)&memptr, ( (alignment >= sizeof(void*)) ? alignment : sizeof(void*) ), n_bytes);
     
-    out_memptr = (status == 0) ? memptr : NULL;
+    out_memptr = (status == 0) ? memptr : nullptr;
     }
   #elif defined(_MSC_VER)
     {
@@ -91,7 +80,7 @@ memory::acquire(const uword n_elem)
     //out_memptr = (eT *) _aligned_malloc( sizeof(eT)*n_elem, 16 );  // lives in malloc.h
     
     const size_t n_bytes   = sizeof(eT)*size_t(n_elem);
-    const size_t alignment = (n_bytes >= size_t(1024)) ? size_t(64) : size_t(16);
+    const size_t alignment = (n_bytes >= size_t(1024)) ? size_t(32) : size_t(16);
     
     out_memptr = (eT *) _aligned_malloc( n_bytes, alignment );
     }
@@ -104,33 +93,19 @@ memory::acquire(const uword n_elem)
   
   // TODO: for mingw, use __mingw_aligned_malloc
   
-  arma_check_bad_alloc( (out_memptr == NULL), "arma::memory::acquire(): out of memory" );
+  arma_check_bad_alloc( (out_memptr == nullptr), "arma::memory::acquire(): out of memory" );
   
   return out_memptr;
   }
 
 
 
-//! get memory in multiples of chunks, holding at least n_elem
-template<typename eT>
-inline
-arma_malloc
-eT*
-memory::acquire_chunked(const uword n_elem)
-  {
-  const uword n_elem_mod = memory::enlarge_to_mult_of_chunksize(n_elem);
-  
-  return memory::acquire<eT>(n_elem_mod);
-  }
-
-
-
 template<typename eT>
 arma_inline
 void
 memory::release(eT* mem)
   {
-  if(mem == NULL)  { return; }
+  if(mem == nullptr)  { return; }
   
   #if   defined(ARMA_USE_TBB_ALLOC)
     {
@@ -236,4 +211,27 @@ memory::mark_as_aligned(const eT*& mem)
 
 
 
+//! NOTE: do not use this function; it will be removed
+inline
+arma_deprecated
+uword
+memory::enlarge_to_mult_of_chunksize(const uword n_elem)   //! NOTE: do not use this function; it will be removed
+  {
+  return n_elem;
+  }
+
+
+
+//! NOTE: do not use this function; it will be removed
+template<typename eT>
+inline
+arma_deprecated
+eT*
+memory::acquire_chunked(const uword n_elem)   //! NOTE: do not use this function; it will be removed
+  {
+  return memory::acquire<eT>(n_elem);
+  }
+
+
+
 //! @}
